@@ -12,15 +12,16 @@ set -eux
 : "${DIST:=}"
 : "${DIST_VERSION:=}"
 
+: "${SKIP_TESTS:=0}"
+
 prepare_build_prerequisites() {
   CMAKE=cmake
 
   case "${OS}-${CPU}" in
-    linux-i386)
-      build_and_install_python
-      ;;
-    linux-*)
-      PREFIX=/usr
+#    linux-i386)                                       #  For i386/Debian (the only 32 bit run) python3 is already installed by
+#      build_and_install_python                        #  linux_install @ install_functions.inc.sh called from install_cacheable_dependencies.sh
+#      ;;                                              #  If there is a distribution that does not have python3 pre-apckeges (highly unlikely)
+    linux-*)                                           #  it shall be implemented like ensure_cmake
       ensure_cmake
       ;;
   esac
@@ -35,7 +36,7 @@ prepare_test_env() {
 
   # update dll search path for windows
   if [[ "${OS}" = "msys" ]]; then
-    export PATH="${LOCAL_BUILDS}/rnp-build/lib:${LOCAL_BUILDS}/rnp-build/bin:${LOCAL_BUILDS}/rnp-build/src/lib:$PATH"
+    export PATH="${LOCAL_BUILDS}/rnp-build/lib:${LOCAL_BUILDS}/rnp-build/bin:${LOCAL_BUILDS}/rnp-build/src/lib:${BOTAN_INSTALL}/bin:$PATH"
   fi
 }
 
@@ -69,7 +70,9 @@ build_tests() {
 }
 
 main() {
-  prepare_test_env
+  if [[ ${SKIP_TESTS} = 0 ]]; then
+    prepare_test_env
+  fi
   prepare_build_prerequisites
 
   export rnpsrc="$PWD"
@@ -78,26 +81,34 @@ main() {
   pushd "${LOCAL_BUILDS}/rnp-build"
 
   cmakeopts=(
-    -DCMAKE_BUILD_TYPE=RelWithDebInfo
+    -DCMAKE_BUILD_TYPE=Release   # RelWithDebInfo -- DebInfo commented out to speed up recurring CI runs.
     -DBUILD_SHARED_LIBS=yes
     -DCMAKE_INSTALL_PREFIX="${RNP_INSTALL}"
     -DCMAKE_PREFIX_PATH="${BOTAN_INSTALL};${JSONC_INSTALL};${GPG_INSTALL}"
   )
-
+  [[ ${SKIP_TESTS} = 1 ]] && cmakeopts+=(-DBUILD_TESTING=OFF)
   [[ "${BUILD_MODE}" = "coverage" ]] && cmakeopts+=(-DENABLE_COVERAGE=yes)
   [[ "${BUILD_MODE}" = "sanitize" ]] && cmakeopts+=(-DENABLE_SANITIZERS=yes)
-  [ -v "GTEST_SOURCES" ] && cmakeopts+=(-DGTEST_SOURCES="${GTEST_SOURCES}")
-  [ -v "DOWNLOAD_GTEST" ] && cmakeopts+=(-DDOWNLOAD_GTEST="${DOWNLOAD_GTEST}")
-  [ -v "DOWNLOAD_RUBYRNP" ] && cmakeopts+=(-DDOWNLOAD_RUBYRNP="${DOWNLOAD_RUBYRNP}")
+  [ -n "${GTEST_SOURCES:-}" ] && cmakeopts+=(-DGTEST_SOURCES="${GTEST_SOURCES}")
+  [ -n "${DOWNLOAD_GTEST:-}" ] && cmakeopts+=(-DDOWNLOAD_GTEST="${DOWNLOAD_GTEST}")
+  [ -n "${DOWNLOAD_RUBYRNP:-}" ] && cmakeopts+=(-DDOWNLOAD_RUBYRNP="${DOWNLOAD_RUBYRNP}")
+  [ -n "${CRYPTO_BACKEND:-}" ] && cmakeopts+=(-DCRYPTO_BACKEND="${CRYPTO_BACKEND}")
+  [ -n "${ENABLE_SM2:-}" ] && cmakeopts+=(-DENABLE_SM2="${ENABLE_SM2}")
+  [ -n "${ENABLE_IDEA:-}" ] && cmakeopts+=(-DENABLE_IDEA="${ENABLE_IDEA}")
+  [ -n "${ENABLE_TWOFISH:-}" ] && cmakeopts+=(-DENABLE_TWOFISH="${ENABLE_TWOFISH}")
+  [ -n "${ENABLE_BRAINPOOL:-}" ] && cmakeopts+=(-DENABLE_BRAINPOOL="${ENABLE_BRAINPOOL}")
 
   if [[ "${OS}" = "msys" ]]; then
     cmakeopts+=(-G "MSYS Makefiles")
   fi
   build_rnp "${rnpsrc}"
-  make_install VERBOSE=1
+  make_install                  # VERBOSE=1 -- verbose flag commented out to speed up recurring CI runs. Uncomment if you are debugging CI
 
-  prepare_tests
-  build_tests
+  if [[ ${SKIP_TESTS} = 0 ]]; then
+    echo "TESTS NOT SKIPPED"
+    prepare_tests
+    build_tests
+  fi
 }
 
 main "$@"
