@@ -103,27 +103,6 @@ bool pgp_generate_subkey(rnp_keygen_subkey_desc_t &     desc,
                          const pgp_password_provider_t &password_provider,
                          pgp_key_store_format_t         secformat);
 
-/** generate a new primary key and subkey
- *
- *  @param primary_desc primary keygen description
- *  @param subkey_desc subkey keygen description
- *  @param merge_defaults true if you want defaults to be set for unset
- *         keygen description parameters.
- *  @param primary_sec pointer to store the generated secret key, must not be NULL
- *  @param primary_pub pointer to store the generated public key, must not be NULL
- *  @param subkey_sec pointer to store the generated secret key, must not be NULL
- *  @param subkey_pub pointer to store the generated public key, must not be NULL
- *  @return true if successful, false otherwise.
- **/
-bool pgp_generate_keypair(rnp_keygen_primary_desc_t &primary_desc,
-                          rnp_keygen_subkey_desc_t & subkey_desc,
-                          bool                       merge_defaults,
-                          pgp_key_t &                primary_sec,
-                          pgp_key_t &                primary_pub,
-                          pgp_key_t &                subkey_sec,
-                          pgp_key_t &                subkey_pub,
-                          pgp_key_store_format_t     secformat);
-
 /**
  * @brief Check two key material for equality. Only public part is checked, so this can be
  *        called on public/secret key material
