@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2020, [Ribose Inc](https://www.ribose.com).
+ * Copyright (c) 2017-2021, [Ribose Inc](https://www.ribose.com).
  * Copyright (c) 2009 The NetBSD Foundation, Inc.
  * All rights reserved.
  *
@@ -37,45 +37,52 @@
 #endif
 #include <string.h>
 #include <stdarg.h>
-#include "../rnp/rnpcfg.h"
-#include "../rnp/fficli.h"
 #include "rnpkeys.h"
-#include "config.h"
+#include "str-utils.h"
 
-// must be placed after include "utils.h"
-#ifndef RNP_USE_STD_REGEX
-#include <regex.h>
-#else
-#include <regex>
-#endif
-
-extern const char *rnp_keys_progname;
-
-const char *usage = "--help OR\n"
-                    "\t--export-key [options] OR\n"
-                    "\t--export-rev [options] OR\n"
-                    "\t--revoke-key [options] OR\n"
-                    "\t--generate-key [options] OR\n"
-                    "\t--import, --import-keys, --import-sigs [options] OR\n"
-                    "\t--list-keys [options] OR\n"
-                    "\t--version\n"
-                    "where options are:\n"
-                    "\t[--cipher=<cipher name>] AND/OR\n"
-                    "\t[--coredumps] AND/OR\n"
-                    "\t[--expert] AND/OR\n"
-                    "\t[--with-sigs] AND/OR\n"
-                    "\t[--force] AND/OR\n"
-                    "\t[--hash=<hash alg>] AND/OR\n"
-                    "\t[--homedir=<homedir>] AND/OR\n"
-                    "\t[--keyring=<keyring>] AND/OR\n"
-                    "\t[--pass-fd=<fd>] OR\n"
-                    "\t[--password=<password>] AND/OR\n"
-                    "\t[--permissive] AND/OR\n"
-                    "\t[--output=file] file OR\n"
-                    "\t[--keystore-format=<format>] AND/OR\n"
-                    "\t[--userid=<userid>] AND/OR\n"
-                    "\t[--rev-type, --rev-reason] AND/OR\n"
-                    "\t[--verbose]\n";
+const char *usage =
+  "Manipulate OpenPGP keys and keyrings.\n"
+  "Usage: rnpkeys --command [options] [files]\n"
+  "Commands:\n"
+  "  -h, --help             This help message.\n"
+  "  -V, --version          Print RNP version information.\n"
+  "  -g, --generate-key     Generate a new keypair (default is RSA).\n"
+  "    --userid             Specify key's userid.\n"
+  "    --expert             Select key type, size, and additional parameters.\n"
+  "    --numbits            Override default key size (2048).\n"
+  "    --expiration         Set key and subkey expiration time.\n"
+  "    --cipher             Set cipher used to encrypt a secret key.\n"
+  "    --hash               Set hash which is used for key derivation.\n"
+  "  -l, --list-keys        List keys in the keyrings.\n"
+  "    --secret             List secret keys instead of public ones.\n"
+  "    --with-sigs          List signatures as well.\n"
+  "  --import               Import keys or signatures.\n"
+  "  --import-keys          Import keys.\n"
+  "  --import-sigs          Import signatures.\n"
+  "    --permissive         Skip erroring keys/sigs instead of failing.\n"
+  "  --export-key           Export a key.\n"
+  "    --secret             Export a secret key instead of a public.\n"
+  "  --export-rev           Export a key's revocation.\n"
+  "    --rev-type           Set revocation type.\n"
+  "    --rev-reason         Human-readable reason for revocation.\n"
+  "  --revoke-key           Revoke a key specified.\n"
+  "  --remove-key           Remove a key specified.\n"
+  "  --edit-key             Edit key properties.\n"
+  "    --check-cv25519-bits Check whether Cv25519 subkey bits are correct.\n"
+  "    --fix-cv25519-bits   Fix Cv25519 subkey bits.\n"
+  "\n"
+  "Other options:\n"
+  "  --homedir              Override home directory (default is ~/.rnp/).\n"
+  "  --password             Password, which should be used during operation.\n"
+  "  --pass-fd              Read password(s) from the file descriptor.\n"
+  "  --force                Force operation (like secret key removal).\n"
+  "  --output [file, -]     Write data to the specified file or stdout.\n"
+  "  --overwrite            Overwrite output file without a prompt.\n"
+  "  --notty                Do not write anything to the TTY.\n"
+  "  --current-time         Override system's time.\n"
+  "\n"
+  "See man page for a detailed listing and explanation.\n"
+  "\n";
 
 struct option options[] = {
   /* key-management commands */
@@ -93,6 +100,8 @@ struct option options[] = {
   {"export-rev", no_argument, NULL, CMD_EXPORT_REV},
   {"export-revocation", no_argument, NULL, CMD_EXPORT_REV},
   {"revoke-key", no_argument, NULL, CMD_REVOKE_KEY},
+  {"remove-key", no_argument, NULL, CMD_REMOVE_KEY},
+  {"edit-key", no_argument, NULL, CMD_EDIT_KEY},
   /* debugging commands */
   {"help", no_argument, NULL, CMD_HELP},
   {"version", no_argument, NULL, CMD_VERSION},
@@ -101,28 +110,30 @@ struct option options[] = {
   {"coredumps", no_argument, NULL, OPT_COREDUMPS},
   {"keystore-format", required_argument, NULL, OPT_KEY_STORE_FORMAT},
   {"userid", required_argument, NULL, OPT_USERID},
-  {"format", required_argument, NULL, OPT_FORMAT},
   {"with-sigs", no_argument, NULL, OPT_WITH_SIGS},
-  {"hash-alg", required_argument, NULL, OPT_HASH_ALG},
   {"hash", required_argument, NULL, OPT_HASH_ALG},
-  {"algorithm", required_argument, NULL, OPT_HASH_ALG},
   {"home", required_argument, NULL, OPT_HOMEDIR},
   {"homedir", required_argument, NULL, OPT_HOMEDIR},
   {"numbits", required_argument, NULL, OPT_NUMBITS},
   {"s2k-iterations", required_argument, NULL, OPT_S2K_ITER},
   {"s2k-msec", required_argument, NULL, OPT_S2K_MSEC},
-  {"verbose", no_argument, NULL, OPT_VERBOSE},
+  {"expiration", required_argument, NULL, OPT_EXPIRATION},
   {"pass-fd", required_argument, NULL, OPT_PASSWDFD},
   {"password", required_argument, NULL, OPT_PASSWD},
   {"results", required_argument, NULL, OPT_RESULTS},
   {"cipher", required_argument, NULL, OPT_CIPHER},
   {"expert", no_argument, NULL, OPT_EXPERT},
   {"output", required_argument, NULL, OPT_OUTPUT},
+  {"overwrite", no_argument, NULL, OPT_OVERWRITE},
   {"force", no_argument, NULL, OPT_FORCE},
   {"secret", no_argument, NULL, OPT_SECRET},
   {"rev-type", required_argument, NULL, OPT_REV_TYPE},
   {"rev-reason", required_argument, NULL, OPT_REV_REASON},
   {"permissive", no_argument, NULL, OPT_PERMISSIVE},
+  {"notty", no_argument, NULL, OPT_NOTTY},
+  {"fix-cv25519-bits", no_argument, NULL, OPT_FIX_25519_BITS},
+  {"check-cv25519-bits", no_argument, NULL, OPT_CHK_25519_BITS},
+  {"current-time", required_argument, NULL, OPT_CURTIME},
   {NULL, 0, NULL, 0},
 };
 
@@ -130,8 +141,8 @@ struct option options[] = {
 static bool
 print_keys_info(cli_rnp_t *rnp, FILE *fp, const char *filter)
 {
-    bool psecret = rnp_cfg_getbool(cli_rnp_cfg(rnp), CFG_SECRET);
-    bool psigs = rnp_cfg_getbool(cli_rnp_cfg(rnp), CFG_WITH_SIGS);
+    bool psecret = rnp->cfg().get_bool(CFG_SECRET);
+    bool psigs = rnp->cfg().get_bool(CFG_WITH_SIGS);
     int  flags = CLI_SEARCH_SUBKEYS_AFTER | (psecret ? CLI_SEARCH_SECRET : 0);
     std::vector<rnp_key_handle_t> keys;
 
@@ -163,21 +174,15 @@ imported_key_changed(json_object *key)
 }
 
 static bool
-import_keys(cli_rnp_t *rnp, const char *file)
+import_keys(cli_rnp_t *rnp, rnp_input_t input, const std::string &inname)
 {
-    rnp_input_t input = NULL;
-    bool        res = false;
-    bool        updated = false;
+    bool res = false;
+    bool updated = false;
 
-    if (rnp_input_from_path(&input, file)) {
-        ERR_MSG("failed to open file %s", file);
-        return false;
-    }
+    uint32_t flags = RNP_LOAD_SAVE_PUBLIC_KEYS | RNP_LOAD_SAVE_SECRET_KEYS |
+                     RNP_LOAD_SAVE_SINGLE | RNP_LOAD_SAVE_BASE64;
 
-    uint32_t flags =
-      RNP_LOAD_SAVE_PUBLIC_KEYS | RNP_LOAD_SAVE_SECRET_KEYS | RNP_LOAD_SAVE_SINGLE;
-
-    bool permissive = rnp_cfg_getbool(cli_rnp_cfg(rnp), CFG_PERMISSIVE);
+    bool permissive = rnp->cfg().get_bool(CFG_PERMISSIVE);
     if (permissive) {
         flags |= RNP_LOAD_SAVE_PERMISSIVE;
     }
@@ -190,8 +195,14 @@ import_keys(cli_rnp_t *rnp, const char *file)
             res = true;
             break;
         }
+        if (ret && updated) {
+            /* some keys were imported, but then error occurred */
+            ERR_MSG("warning: not all data was processed.");
+            res = true;
+            break;
+        }
         if (ret) {
-            ERR_MSG("failed to import key(s), from file %s, stopping.", file);
+            ERR_MSG("failed to import key(s) from %s, stopping.", inname.c_str());
             break;
         }
 
@@ -228,8 +239,8 @@ import_keys(cli_rnp_t *rnp, const char *file)
 
     if (updated) {
         // set default key if we didn't have one
-        if (cli_rnp_defkey(rnp).empty()) {
-            cli_rnp_set_default_key(rnp);
+        if (rnp->defkey().empty()) {
+            rnp->set_defkey();
         }
 
         // save public and secret keyrings
@@ -237,21 +248,13 @@ import_keys(cli_rnp_t *rnp, const char *file)
             ERR_MSG("failed to save keyrings");
         }
     }
-    rnp_input_destroy(input);
     return res;
 }
 
 static bool
-import_sigs(cli_rnp_t *rnp, const char *file)
+import_sigs(cli_rnp_t *rnp, rnp_input_t input, const std::string &inname)
 {
-    rnp_input_t input = NULL;
-    bool        res = false;
-
-    if (rnp_input_from_path(&input, file)) {
-        ERR_MSG("Failed to open file %s", file);
-        return false;
-    }
-
+    bool         res = false;
     char *       results = NULL;
     json_object *jso = NULL;
     json_object *sigs = NULL;
@@ -260,7 +263,7 @@ import_sigs(cli_rnp_t *rnp, const char *file)
     int          old_sigs = 0;
 
     if (rnp_import_signatures(rnp->ffi, input, 0, &results)) {
-        ERR_MSG("Failed to import signatures from file %s", file);
+        ERR_MSG("Failed to import signatures from %s", inname.c_str());
         goto done;
     }
     // print information about imported signature(s)
@@ -305,105 +308,90 @@ import_sigs(cli_rnp_t *rnp, const char *file)
 done:
     json_object_put(jso);
     rnp_buffer_destroy(results);
-    rnp_input_destroy(input);
     return res;
 }
 
 static bool
-import(cli_rnp_t *rnp, const char *file, int cmd)
+import(cli_rnp_t *rnp, const std::string &spec, int cmd)
 {
-    if (!file) {
-        ERR_MSG("Import file isn't specified");
+    if (spec.empty()) {
+        ERR_MSG("Import path isn't specified");
         return false;
     }
-
-    if (cmd == CMD_IMPORT_KEYS) {
-        return import_keys(rnp, file);
-    }
-    if (cmd == CMD_IMPORT_SIGS) {
-        return import_sigs(rnp, file);
-    }
-
-    rnp_input_t input = NULL;
-    if (rnp_input_from_path(&input, file)) {
-        ERR_MSG("Failed to open file %s", file);
+    rnp_input_t input = cli_rnp_input_from_specifier(*rnp, spec, NULL);
+    if (!input) {
+        ERR_MSG("Failed to create input for %s", spec.c_str());
         return false;
     }
+    if (cmd == CMD_IMPORT) {
+        char *contents = NULL;
+        if (rnp_guess_contents(input, &contents)) {
+            ERR_MSG("Warning! Failed to guess content type to import. Assuming keys.");
+        }
+        cmd = (contents && !strcmp(contents, "signature")) ? CMD_IMPORT_SIGS : CMD_IMPORT_KEYS;
+        rnp_buffer_destroy(contents);
+    }
 
-    char *contents = NULL;
-    if (rnp_guess_contents(input, &contents)) {
-        ERR_MSG("Warning! Failed to guess content type to import. Assuming keys.");
+    bool res = false;
+    switch (cmd) {
+    case CMD_IMPORT_KEYS:
+        res = import_keys(rnp, input, spec);
+        break;
+    case CMD_IMPORT_SIGS:
+        res = import_sigs(rnp, input, spec);
+        break;
+    default:
+        ERR_MSG("Unexpected command: %d", cmd);
     }
     rnp_input_destroy(input);
-    bool signature = contents && !strcmp(contents, "signature");
-    rnp_buffer_destroy(contents);
-
-    return signature ? import_sigs(rnp, file) : import_keys(rnp, file);
-}
-
-void
-print_praise(void)
-{
-    ERR_MSG("%s\nAll bug reports, praise and chocolate, please, to:\n%s",
-            PACKAGE_STRING,
-            PACKAGE_BUGREPORT);
+    return res;
 }
 
 /* print a usage message */
 void
 print_usage(const char *usagemsg)
 {
-    print_praise();
-    ERR_MSG("Usage: %s %s", rnp_keys_progname, usagemsg);
+    cli_rnp_print_praise();
+    ERR_MSG("%s", usagemsg);
 }
 
 /* do a command once for a specified file 'f' */
 bool
 rnp_cmd(cli_rnp_t *rnp, optdefs_t cmd, const char *f)
 {
-    const char *key;
     std::string fs;
 
     switch (cmd) {
     case CMD_LIST_KEYS:
-        if (!f) {
-            list *ids = NULL;
-            if ((ids = rnp_cfg_getlist(cli_rnp_cfg(rnp), CFG_USERID)) &&
-                list_length(*ids) > 0) {
-                f = (fs = rnp_cfg_getlist_string(cli_rnp_cfg(rnp), CFG_USERID, 0)).c_str();
-            }
+        if (!f && rnp->cfg().get_count(CFG_USERID)) {
+            fs = rnp->cfg().get_str(CFG_USERID, 0);
+            f = fs.c_str();
         }
         return print_keys_info(rnp, stdout, f);
     case CMD_EXPORT_KEY: {
-        key = f;
-        if (!key) {
-            list *ids = NULL;
-            if ((ids = rnp_cfg_getlist(cli_rnp_cfg(rnp), CFG_USERID)) &&
-                list_length(*ids) > 0) {
-                f = (fs = rnp_cfg_getlist_string(cli_rnp_cfg(rnp), CFG_USERID, 0)).c_str();
-            }
+        if (!f && rnp->cfg().get_count(CFG_USERID)) {
+            fs = rnp->cfg().get_str(CFG_USERID, 0);
+            f = fs.c_str();
         }
-        if (!key) {
-            ERR_MSG("key '%s' not found", f);
+        if (!f) {
+            ERR_MSG("No key specified.");
             return 0;
         }
-        return cli_rnp_export_keys(rnp, key);
+        return cli_rnp_export_keys(rnp, f);
     }
     case CMD_IMPORT:
     case CMD_IMPORT_KEYS:
     case CMD_IMPORT_SIGS:
-        return import(rnp, f, cmd);
+        return import(rnp, f ? f : "", cmd);
     case CMD_GENERATE_KEY: {
-        if (f == NULL) {
-            list *ids = NULL;
-            if ((ids = rnp_cfg_getlist(cli_rnp_cfg(rnp), CFG_USERID)) &&
-                list_length(*ids) > 0) {
-                if (list_length(*ids) == 1) {
-                    f = (fs = rnp_cfg_getlist_string(cli_rnp_cfg(rnp), CFG_USERID, 0)).c_str();
-                } else {
-                    ERR_MSG("Only single userid is supported for generated keys");
-                    return false;
-                }
+        if (!f) {
+            size_t count = rnp->cfg().get_count(CFG_USERID);
+            if (count == 1) {
+                fs = rnp->cfg().get_str(CFG_USERID, 0);
+                f = fs.c_str();
+            } else if (count > 1) {
+                ERR_MSG("Only single userid is supported for generated keys");
+                return false;
             }
         }
         return cli_rnp_generate_key(rnp, f);
@@ -422,179 +410,138 @@ rnp_cmd(cli_rnp_t *rnp, optdefs_t cmd, const char *f)
         }
         return cli_rnp_revoke_key(rnp, f);
     }
+    case CMD_REMOVE_KEY: {
+        if (!f) {
+            ERR_MSG("You need to specify key or subkey to remove.");
+            return false;
+        }
+        return cli_rnp_remove_key(rnp, f);
+    }
+    case CMD_EDIT_KEY: {
+        if (!f) {
+            ERR_MSG("You need to specify a key or subkey to edit.");
+            return false;
+        }
+        return rnp->edit_key(f);
+    }
     case CMD_VERSION:
-        print_praise();
+        cli_rnp_print_praise();
         return true;
     case CMD_HELP:
     default:
         print_usage(usage);
-        return false;
+        return true;
     }
 }
 
 /* set the option */
 bool
-setoption(rnp_cfg_t *cfg, optdefs_t *cmd, int val, const char *arg)
+setoption(rnp_cfg &cfg, optdefs_t *cmd, int val, const char *arg)
 {
-    bool ret = false;
-
     switch (val) {
     case OPT_COREDUMPS:
-        ret = rnp_cfg_setbool(cfg, CFG_COREDUMPS, true);
-        break;
+#ifdef _WIN32
+        ERR_MSG("warning: --coredumps doesn't make sense on windows systems.");
+#endif
+        cfg.set_bool(CFG_COREDUMPS, true);
+        return true;
     case CMD_GENERATE_KEY:
-        ret = rnp_cfg_setbool(cfg, CFG_NEEDSSECKEY, true);
+        cfg.set_bool(CFG_NEEDSSECKEY, true);
         *cmd = (optdefs_t) val;
-        break;
+        return true;
     case OPT_EXPERT:
-        ret = rnp_cfg_setbool(cfg, CFG_EXPERT, true);
-        break;
+        cfg.set_bool(CFG_EXPERT, true);
+        return true;
     case CMD_LIST_KEYS:
     case CMD_EXPORT_KEY:
     case CMD_EXPORT_REV:
     case CMD_REVOKE_KEY:
+    case CMD_REMOVE_KEY:
+    case CMD_EDIT_KEY:
     case CMD_IMPORT:
     case CMD_IMPORT_KEYS:
     case CMD_IMPORT_SIGS:
     case CMD_HELP:
     case CMD_VERSION:
         *cmd = (optdefs_t) val;
-        ret = true;
-        break;
+        return true;
     /* options */
     case OPT_KEY_STORE_FORMAT:
-        if (arg == NULL) {
-            ERR_MSG("No keyring format argument provided");
-            break;
-        }
-        ret = rnp_cfg_setstr(cfg, CFG_KEYSTOREFMT, arg);
-        break;
+        cfg.set_str(CFG_KEYSTOREFMT, arg);
+        return true;
     case OPT_USERID:
-        if (arg == NULL) {
-            ERR_MSG("no userid argument provided");
-            break;
-        }
-        ret = rnp_cfg_addstr(cfg, CFG_USERID, arg);
-        break;
-    case OPT_VERBOSE:
-        ret = rnp_cfg_setint(cfg, CFG_VERBOSE, rnp_cfg_getint(cfg, CFG_VERBOSE) + 1);
-        break;
+        cfg.add_str(CFG_USERID, arg);
+        return true;
     case OPT_HOMEDIR:
-        if (arg == NULL) {
-            ERR_MSG("no home directory argument provided");
-            break;
-        }
-        ret = rnp_cfg_setstr(cfg, CFG_HOMEDIR, arg);
-        break;
+        cfg.set_str(CFG_HOMEDIR, arg);
+        return true;
     case OPT_NUMBITS: {
-        if (arg == NULL) {
-            ERR_MSG("no number of bits argument provided");
-            break;
-        }
-        int bits = atoi(arg);
-        if ((bits < 1024) || (bits > 16384)) {
+        int bits = 0;
+        if (!rnp::str_to_int(arg, bits) || (bits < 1024) || (bits > 16384)) {
             ERR_MSG("wrong bits value: %s", arg);
-            break;
+            return false;
         }
-        ret = rnp_cfg_setint(cfg, CFG_NUMBITS, bits);
-        break;
-    }
-    case OPT_HASH_ALG: {
-        if (arg == NULL) {
-            ERR_MSG("No hash algorithm argument provided");
-            break;
-        }
-        bool supported = false;
-        if (rnp_supports_feature("hash algorithm", arg, &supported) || !supported) {
-            ERR_MSG("Unsupported hash algorithm: %s", arg);
-            break;
-        }
-        ret = rnp_cfg_setstr(cfg, CFG_HASH, arg);
-        break;
+        cfg.set_int(CFG_NUMBITS, bits);
+        return true;
     }
+    case OPT_HASH_ALG:
+        return cli_rnp_set_hash(cfg, arg);
     case OPT_S2K_ITER: {
-        if (arg == NULL) {
-            ERR_MSG("No s2k iteration argument provided");
-            break;
-        }
         int iterations = atoi(arg);
         if (!iterations) {
             ERR_MSG("Wrong iterations value: %s", arg);
-            break;
-        }
-        ret = rnp_cfg_setint(cfg, CFG_S2K_ITER, iterations);
-        break;
-    }
-    case OPT_S2K_MSEC: {
-        if (arg == NULL) {
-            ERR_MSG("No s2k msec argument provided");
-            break;
-        }
-        int msec = atoi(arg);
-        if (!msec) {
-            ERR_MSG("Invalid s2k msec value: %s", arg);
-            break;
-        }
-        ret = rnp_cfg_setint(cfg, CFG_S2K_MSEC, atoi(arg));
-        break;
-    }
-    case OPT_PASSWDFD:
-        if (arg == NULL) {
-            ERR_MSG("no pass-fd argument provided");
-            break;
-        }
-        ret = rnp_cfg_setstr(cfg, CFG_PASSFD, arg);
-        break;
-    case OPT_PASSWD:
-        if (arg == NULL) {
-            ERR_MSG("No password argument provided");
             return false;
         }
-        ret = rnp_cfg_setstr(cfg, CFG_PASSWD, arg);
-        break;
-    case OPT_RESULTS:
-        if (arg == NULL) {
-            ERR_MSG("No output filename argument provided");
-            break;
-        }
-        ret = rnp_cfg_setstr(cfg, CFG_IO_RESS, arg);
-        break;
-    case OPT_FORMAT:
-        ret = rnp_cfg_setstr(cfg, CFG_KEYFORMAT, arg);
-        break;
-    case OPT_CIPHER: {
-        bool supported = false;
-        if (rnp_supports_feature("symmetric algorithm", arg, &supported) || !supported) {
-            ERR_MSG("Unsupported symmetric algorithm: %s", arg);
-            break;
-        }
-        ret = rnp_cfg_setstr(cfg, CFG_CIPHER, arg);
-        break;
+        cfg.set_int(CFG_S2K_ITER, iterations);
+        return true;
     }
+    case OPT_EXPIRATION:
+        cfg.set_str(CFG_KG_PRIMARY_EXPIRATION, arg);
+        cfg.set_str(CFG_KG_SUBKEY_EXPIRATION, arg);
+        return true;
+    case OPT_S2K_MSEC: {
+        int msec = 0;
+        if (!rnp::str_to_int(arg, msec) || !msec) {
+            ERR_MSG("Invalid s2k msec value: %s", arg);
+            return false;
+        }
+        cfg.set_int(CFG_S2K_MSEC, msec);
+        return true;
+    }
+    case OPT_PASSWDFD:
+        cfg.set_str(CFG_PASSFD, arg);
+        return true;
+    case OPT_PASSWD:
+        cfg.set_str(CFG_PASSWD, arg);
+        return true;
+    case OPT_RESULTS:
+        cfg.set_str(CFG_IO_RESS, arg);
+        return true;
+    case OPT_CIPHER:
+        return cli_rnp_set_cipher(cfg, arg);
     case OPT_DEBUG:
-        ret = !rnp_enable_debug(arg);
-        break;
+        ERR_MSG("Option --debug is deprecated, ignoring.");
+        return true;
     case OPT_OUTPUT:
-        if (arg == NULL) {
-            ERR_MSG("No output filename argument provided");
-            break;
-        }
-        ret = rnp_cfg_setstr(cfg, CFG_OUTFILE, arg);
-        break;
-    case OPT_FORCE:
-        ret = rnp_cfg_setbool(cfg, CFG_FORCE, true);
-        break;
-    case OPT_SECRET:
-        ret = rnp_cfg_setbool(cfg, CFG_SECRET, true);
-        break;
-    case OPT_WITH_SIGS:
-        ret = rnp_cfg_setbool(cfg, CFG_WITH_SIGS, true);
-        break;
-    case OPT_REV_TYPE: {
         if (!arg) {
-            ERR_MSG("No revocation type argument provided");
-            break;
+            ERR_MSG("No output filename argument provided");
+            return false;
         }
+        cfg.set_str(CFG_OUTFILE, arg);
+        return true;
+    case OPT_OVERWRITE:
+        cfg.set_bool(CFG_OVERWRITE, true);
+        return true;
+    case OPT_FORCE:
+        cfg.set_bool(CFG_FORCE, true);
+        return true;
+    case OPT_SECRET:
+        cfg.set_bool(CFG_SECRET, true);
+        return true;
+    case OPT_WITH_SIGS:
+        cfg.set_bool(CFG_WITH_SIGS, true);
+        return true;
+    case OPT_REV_TYPE: {
         std::string revtype = arg;
         if (revtype == "0") {
             revtype = "no";
@@ -605,113 +552,57 @@ setoption(rnp_cfg_t *cfg, optdefs_t *cmd, int val, const char *arg)
         } else if (revtype == "3") {
             revtype = "retired";
         }
-        ret = rnp_cfg_setstr(cfg, CFG_REV_TYPE, revtype.c_str());
-        break;
+        cfg.set_str(CFG_REV_TYPE, revtype);
+        return true;
     }
     case OPT_REV_REASON:
-        ret = rnp_cfg_setstr(cfg, CFG_REV_REASON, arg);
-        break;
+        cfg.set_str(CFG_REV_REASON, arg);
+        return true;
     case OPT_PERMISSIVE:
-        ret = rnp_cfg_setbool(cfg, CFG_PERMISSIVE, true);
-        break;
+        cfg.set_bool(CFG_PERMISSIVE, true);
+        return true;
+    case OPT_NOTTY:
+        cfg.set_bool(CFG_NOTTY, true);
+        return true;
+    case OPT_FIX_25519_BITS:
+        cfg.set_bool(CFG_FIX_25519_BITS, true);
+        return true;
+    case OPT_CHK_25519_BITS:
+        cfg.set_bool(CFG_CHK_25519_BITS, true);
+        return true;
+    case OPT_CURTIME:
+        cfg.set_str(CFG_CURTIME, arg);
+        return true;
     default:
         *cmd = CMD_HELP;
-        ret = true;
-        break;
+        return true;
     }
-    return ret;
-}
-
-/* we have -o option=value -- parse, and process */
-bool
-parse_option(rnp_cfg_t *cfg, optdefs_t *cmd, const char *s)
-{
-#ifndef RNP_USE_STD_REGEX
-    static regex_t opt;
-    struct option *op;
-    static int     compiled;
-    regmatch_t     matches[10];
-    char           option[128];
-    char           value[128];
-
-    if (!compiled) {
-        compiled = 1;
-        if (regcomp(&opt, "([^=]{1,128})(=(.*))?", REG_EXTENDED) != 0) {
-            ERR_MSG("Can't compile regex");
-            return false;
-        }
-    }
-    if (regexec(&opt, s, 10, matches, 0) == 0) {
-        (void) snprintf(option,
-                        sizeof(option),
-                        "%.*s",
-                        (int) (matches[1].rm_eo - matches[1].rm_so),
-                        &s[matches[1].rm_so]);
-        if (matches[2].rm_so > 0) {
-            (void) snprintf(value,
-                            sizeof(value),
-                            "%.*s",
-                            (int) (matches[3].rm_eo - matches[3].rm_so),
-                            &s[matches[3].rm_so]);
-        } else {
-            value[0] = 0x0;
-        }
-        for (op = options; op->name; op++) {
-            if (strcmp(op->name, option) == 0) {
-                return setoption(cfg, cmd, op->val, value);
-            }
-        }
-    }
-#else
-    static std::regex re("([^=]{1,128})(=(.*))?", std::regex_constants::extended);
-    std::string       input = s;
-    std::smatch       result;
-
-    if (std::regex_match(input, result, re)) {
-        std::string option = result[1];
-        std::string value;
-        if (result.size() >= 4) {
-            value = result[3];
-        }
-        for (struct option *op = options; op->name; op++) {
-            if (strcmp(op->name, option.c_str()) == 0) {
-                return setoption(cfg, cmd, op->val, value.c_str());
-            }
-        }
-    }
-#endif
-    return false;
 }
 
 bool
-rnpkeys_init(cli_rnp_t *rnp, const rnp_cfg_t *cfg)
+rnpkeys_init(cli_rnp_t *rnp, const rnp_cfg &cfg)
 {
-    rnp_cfg_t rnpcfg = {};
-    bool      ret = false;
-    rnp_cfg_init(&rnpcfg);
-    rnp_cfg_load_defaults(&rnpcfg);
-    rnp_cfg_setint(&rnpcfg, CFG_NUMBITS, DEFAULT_RSA_NUMBITS);
-    rnp_cfg_setstr(&rnpcfg, CFG_IO_RESS, "<stdout>");
-    rnp_cfg_setstr(&rnpcfg, CFG_KEYFORMAT, "human");
-    if (!rnp_cfg_copy(&rnpcfg, cfg)) {
-        ERR_MSG("fatal: out of memory");
-        goto end;
-    }
-    if (!cli_cfg_set_keystore_info(&rnpcfg)) {
+    rnp_cfg rnpcfg;
+    bool    ret = false;
+    rnpcfg.load_defaults();
+    rnpcfg.set_int(CFG_NUMBITS, DEFAULT_RSA_NUMBITS);
+    rnpcfg.set_str(CFG_IO_RESS, "<stdout>");
+    rnpcfg.copy(cfg);
+
+    if (!cli_cfg_set_keystore_info(rnpcfg)) {
         ERR_MSG("fatal: cannot set keystore info");
         goto end;
     }
-    if (!cli_rnp_init(rnp, &rnpcfg)) {
+    if (!rnp->init(rnpcfg)) {
         ERR_MSG("fatal: failed to initialize rnpkeys");
         goto end;
     }
     /* TODO: at some point we should check for error here */
-    (void) cli_rnp_load_keyrings(rnp, true);
+    (void) rnp->load_keyrings(true);
     ret = true;
 end:
-    rnp_cfg_free(&rnpcfg);
     if (!ret) {
-        cli_rnp_end(rnp);
+        rnp->end();
     }
     return ret;
 }
