@@ -57,62 +57,92 @@ TEST_F(rnp_tests, test_key_validate)
     rnp_key_store_t *secring;
     pgp_key_t *      key = NULL;
 
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/1/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/1/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     /* this keyring has one expired subkey */
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "1d7e8a5393c997a8", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "1d7e8a5393c997a8"));
     assert_false(key->valid());
     assert_true(key->expired());
     assert_true(all_keys_valid(pubring, key));
     delete pubring;
 
-    /* secret key doesn't have expired binding signature so considered as valid */
-    secring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/1/secring.gpg");
+    /* secret key is marked is expired as well */
+    secring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/1/secring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(secring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(secring, "1d7e8a5393c997a8", NULL));
-    assert_true(key->valid());
-    assert_false(key->expired());
-    assert_true(all_keys_valid(secring));
+    assert_non_null(key = rnp_tests_get_key_by_id(secring, "1d7e8a5393c997a8"));
+    assert_false(key->valid());
+    assert_true(key->expired());
+    assert_true(all_keys_valid(secring, key));
     delete secring;
 
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/2/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/2/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_true(all_keys_valid(pubring));
+
+    /* secret keyring doesn't have signatures - so keys are marked as invalid */
+    secring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/2/secring.gpg", global_ctx);
+    assert_true(rnp_key_store_load_from_path(secring, NULL));
+    assert_false(all_keys_valid(secring));
+    /* but after adding signatures from public it is marked as valid */
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "dc70c124a50283f1"));
+    assert_non_null(rnp_key_store_import_key(secring, key, true, NULL));
+    assert_true(all_keys_valid(secring));
     delete pubring;
-
-    secring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/2/secring.gpg");
-    assert_true(rnp_key_store_load_from_path(secring, NULL));
-    assert_true(all_keys_valid(secring));
     delete secring;
 
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_KBX, "data/keyrings/3/pubring.kbx");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_KBX, "data/keyrings/3/pubring.kbx", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_true(all_keys_valid(pubring));
 
-    secring = new rnp_key_store_t(PGP_KEY_STORE_G10, "data/keyrings/3/private-keys-v1.d");
-    pgp_key_provider_t key_provider = {.callback = rnp_key_provider_store,
-                                       .userdata = pubring};
+    secring =
+      new rnp_key_store_t(PGP_KEY_STORE_G10, "data/keyrings/3/private-keys-v1.d", global_ctx);
+    pgp_key_provider_t key_provider(rnp_key_provider_store, pubring);
     assert_true(rnp_key_store_load_from_path(secring, &key_provider));
     assert_true(all_keys_valid(secring));
     delete pubring;
     delete secring;
 
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/4/pubring.pgp");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/4/pubring.pgp", global_ctx);
+    assert_true(rnp_key_store_load_from_path(pubring, NULL));
+    /* certification has signature with MD5 hash algorithm */
+    assert_false(all_keys_valid(pubring));
+
+    rnp_key_store_clear(pubring);
+    /* add rule which allows MD5 */
+    rnp::SecurityRule allow_md5(
+      rnp::FeatureType::Hash, PGP_HASH_MD5, rnp::SecurityLevel::Default);
+    allow_md5.override = true;
+    global_ctx.profile.add_rule(allow_md5);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_true(all_keys_valid(pubring));
+    rnp_key_store_clear(pubring);
+    /* remove rule */
+    assert_true(global_ctx.profile.del_rule(allow_md5));
+    assert_true(rnp_key_store_load_from_path(pubring, NULL));
+    assert_false(all_keys_valid(pubring));
     delete pubring;
 
-    secring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/4/secring.pgp");
+    /* secret keyring doesn't have certifications - so marked as invalid */
+    secring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/4/secring.pgp", global_ctx);
     assert_true(rnp_key_store_load_from_path(secring, NULL));
-    assert_true(all_keys_valid(secring));
+    assert_false(all_keys_valid(secring));
     delete secring;
 
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/5/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/5/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_true(all_keys_valid(pubring));
     delete pubring;
 
-    secring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/5/secring.gpg");
+    secring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, "data/keyrings/5/secring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(secring, NULL));
     assert_true(all_keys_valid(secring));
     delete secring;
@@ -127,7 +157,7 @@ key_store_add(rnp_key_store_t *keyring, const char *keypath)
     pgp_transferable_key_t tkey = {};
 
     assert_rnp_success(init_file_src(&keysrc, keypath));
-    assert_rnp_success(process_pgp_key(&keysrc, tkey, false));
+    assert_rnp_success(process_pgp_key(keysrc, tkey, false));
     assert_true(rnp_key_store_add_transferable_key(keyring, &tkey));
     src_close(&keysrc);
 }
@@ -135,7 +165,7 @@ key_store_add(rnp_key_store_t *keyring, const char *keypath)
 static bool
 key_check(rnp_key_store_t *keyring, const std::string &keyid, bool valid, bool expired = false)
 {
-    pgp_key_t *key = rnp_tests_get_key_by_id(keyring, keyid, NULL);
+    pgp_key_t *key = rnp_tests_get_key_by_id(keyring, keyid);
     return key && (key->validated()) && (key->valid() == valid) && (key->expired() == expired);
 }
 
@@ -144,7 +174,7 @@ TEST_F(rnp_tests, test_forged_key_validate)
     rnp_key_store_t *pubring;
     pgp_key_t *      key = NULL;
 
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "");
+    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, "", global_ctx);
 
     /* load valid dsa-eg key */
     key_store_add(pubring, DATA_PATH "dsa-eg-pub.pgp");
@@ -160,10 +190,10 @@ TEST_F(rnp_tests, test_forged_key_validate)
 
     /* load dsa-eg key with forged key material */
     key_store_add(pubring, DATA_PATH "dsa-eg-pub-forged-material.pgp");
-    key = rnp_tests_get_key_by_id(pubring, "C8A10A7D78273E10", NULL);
+    key = rnp_tests_get_key_by_id(pubring, "C8A10A7D78273E10");
     assert_null(key);
     /* malformed key material causes keyid change */
-    key = rnp_tests_get_key_by_id(pubring, "C258AB3B54097B9B", NULL);
+    key = rnp_tests_get_key_by_id(pubring, "C258AB3B54097B9B");
     assert_non_null(key);
     assert_false(key->valid());
     assert_false(key->expired());
@@ -187,7 +217,7 @@ TEST_F(rnp_tests, test_forged_key_validate)
 
     /* load eddsa key with forged key material */
     key_store_add(pubring, DATA_PATH "ecc-25519-pub-forged-material.pgp");
-    key = rnp_tests_get_key_by_id(pubring, "1BEF78DF765B79A2", NULL);
+    key = rnp_tests_get_key_by_id(pubring, "1BEF78DF765B79A2");
     assert_non_null(key);
     assert_false(key->valid());
     assert_false(key->expired());
@@ -208,9 +238,9 @@ TEST_F(rnp_tests, test_forged_key_validate)
 
     /* load ecdsa/ecdh key with forged key material. Subkey is not valid as well. */
     key_store_add(pubring, DATA_PATH "ecc-p256-pub-forged-material.pgp");
-    key = rnp_tests_get_key_by_id(pubring, "23674F21B2441527", NULL);
+    key = rnp_tests_get_key_by_id(pubring, "23674F21B2441527");
     assert_null(key);
-    key = rnp_tests_get_key_by_id(pubring, "41DEA786D18E5184", NULL);
+    key = rnp_tests_get_key_by_id(pubring, "41DEA786D18E5184");
     assert_non_null(key);
     assert_false(key->valid());
     assert_false(key->expired());
@@ -243,9 +273,17 @@ TEST_F(rnp_tests, test_forged_key_validate)
 
     /* load valid rsa/rsa keypair */
     key_store_add(pubring, DATA_PATH "rsa-rsa-pub.pgp");
+    /* it is valid only till year 2024 since SHA1 hash is used for signatures */
     assert_true(key_check(pubring, "2FB9179118898E8B", true));
     assert_true(key_check(pubring, "6E2F73008F8B8D6E", true));
     rnp_key_store_clear(pubring);
+    /* load eddsa key which uses SHA1 signature and is created after the cutoff date */
+    global_ctx.set_time(SHA1_KEY_FROM + 10);
+    key_store_add(pubring, DATA_PATH "eddsa-2024-pub.pgp");
+    assert_false(key_check(pubring, "980E3741F632212C", true));
+    assert_false(key_check(pubring, "6DA00BF7F8B59B53", true));
+    global_ctx.set_time(0);
+    rnp_key_store_clear(pubring);
 
     /* load rsa/rsa key with forged self-signature. Valid because of valid binding. */
     key_store_add(pubring, DATA_PATH "rsa-rsa-pub-forged-key.pgp");
@@ -255,9 +293,9 @@ TEST_F(rnp_tests, test_forged_key_validate)
 
     /* load rsa/rsa key with forged key material. Subkey is not valid as well. */
     key_store_add(pubring, DATA_PATH "rsa-rsa-pub-forged-material.pgp");
-    key = rnp_tests_get_key_by_id(pubring, "2FB9179118898E8B", NULL);
+    key = rnp_tests_get_key_by_id(pubring, "2FB9179118898E8B");
     assert_null(key);
-    key = rnp_tests_get_key_by_id(pubring, "791B14952D8F906C", NULL);
+    key = rnp_tests_get_key_by_id(pubring, "791B14952D8F906C");
     assert_non_null(key);
     assert_false(key->valid());
     assert_false(key->expired());
@@ -317,7 +355,8 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice is signed by Basil, but without the Basil's key.
      * Result: Alice [valid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case1/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case1/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_non_null(key = rnp_tests_key_search(pubring, "Alice <alice@rnp>"));
     assert_true(key->valid());
@@ -330,9 +369,10 @@ TEST_F(rnp_tests, test_key_validity)
      * corrupted.
      * Result: Alice [invalid], Basil [valid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case2/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case2/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_false(key->valid());
     assert_false(key->expired());
     assert_non_null(key = rnp_tests_key_search(pubring, "Basil <basil@rnp>"));
@@ -345,9 +385,10 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice is signed by Basil, but doesn't have self-signature
      * Result: Alice [invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case3/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case3/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_false(key->valid());
     assert_false(key->expired());
     assert_non_null(key = rnp_tests_key_search(pubring, "Basil <basil@rnp>"));
@@ -360,7 +401,8 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice subkey has invalid binding signature
      * Result: Alice [valid], Alice sub [invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case4/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case4/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_non_null(key = rnp_tests_key_search(pubring, "Alice <alice@rnp>"));
     assert_true(key->valid());
@@ -380,7 +422,8 @@ TEST_F(rnp_tests, test_key_validity)
      * Note: to re-generate keyring file, use generate.cpp from case5 folder.
      *       To build it, feed -DBUILD_TESTING_GENERATORS=On to the cmake.
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case5/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case5/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_non_null(key = rnp_tests_key_search(pubring, "Alice <alice@rnp>"));
     assert_true(key->valid());
@@ -396,7 +439,8 @@ TEST_F(rnp_tests, test_key_validity)
      * Key Alice has revocation signature by Alice, and subkey doesn't
      * Result: Alice [invalid], Alice sub [invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case6/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case6/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_non_null(key = rnp_tests_key_search(pubring, "Alice <alice@rnp>"));
     assert_false(key->valid());
@@ -414,7 +458,8 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice subkey has revocation signature by Alice
      * Result: Alice [valid], Alice sub [invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case7/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case7/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_non_null(key = rnp_tests_key_search(pubring, "Alice <alice@rnp>"));
     assert_true(key->valid());
@@ -432,9 +477,10 @@ TEST_F(rnp_tests, test_key_validity)
      * Userid is stripped from the key, but it still has valid subkey binding
      * Result: Alice [valid], Alice sub[valid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case8/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case8/pubring.gpg", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_true(key->valid());
     assert_int_equal(key->subkey_count(), 1);
     assert_non_null(subkey = pgp_key_get_subkey(key, pubring, 0));
@@ -447,10 +493,11 @@ TEST_F(rnp_tests, test_key_validity)
      * expiration.
      * Result: Alice [valid], Alice sub[valid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case9/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case9/pubring.gpg", global_ctx);
     assert_non_null(pubring);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_true(key->valid());
     assert_false(key->expired());
     assert_int_equal(key->subkey_count(), 1);
@@ -464,10 +511,11 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice key has expiring direct-key signature and non-expiring self-certification.
      * Result: Alice [invalid], Alice sub[invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case10/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case10/pubring.gpg", global_ctx);
     assert_non_null(pubring);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_false(key->valid());
     assert_true(key->expired());
     assert_int_equal(key->subkey_count(), 1);
@@ -481,10 +529,11 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice key has expiring direct-key signature, non-expiring self-certification and
      * expiring primary userid certification. Result: Alice [invalid], Alice sub[invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case11/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case11/pubring.gpg", global_ctx);
     assert_non_null(pubring);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_false(key->valid());
     assert_true(key->expired());
     assert_int_equal(key->expiration(), 100);
@@ -499,10 +548,11 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice key has non-expiring direct-key signature, non-expiring self-certification and
      * expiring primary userid certification. Result: Alice [invalid], Alice sub[invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case12/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case12/pubring.gpg", global_ctx);
     assert_non_null(pubring);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_false(key->valid());
     assert_true(key->expired());
     assert_int_equal(key->expiration(), 2000);
@@ -517,10 +567,11 @@ TEST_F(rnp_tests, test_key_validity)
      * Alice key has expiring direct-key signature, non-expiring self-certification and
      * non-expiring primary userid certification. Result: Alice [invalid], Alice sub[invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case13/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case13/pubring.gpg", global_ctx);
     assert_non_null(pubring);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_false(key->valid());
     assert_true(key->expired());
     assert_int_equal(key->expiration(), 6);
@@ -536,10 +587,11 @@ TEST_F(rnp_tests, test_key_validity)
      * non-expiring primary userid certification (with 0 key expiration subpacket). Result:
      * Alice [invalid], Alice sub[invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case14/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case14/pubring.gpg", global_ctx);
     assert_non_null(pubring);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "0451409669FFDE3C"));
     assert_false(key->valid());
     assert_true(key->expired());
     assert_int_equal(key->expiration(), 6);
@@ -554,10 +606,11 @@ TEST_F(rnp_tests, test_key_validity)
      * Signing subkey has expired primary-key signature embedded into the subkey binding.
      * Result: primary [valid], sub[invalid]
      */
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case15/pubring.gpg");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "case15/pubring.gpg", global_ctx);
     assert_non_null(pubring);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
-    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "E863072D3E9042EE", NULL));
+    assert_non_null(key = rnp_tests_get_key_by_id(pubring, "E863072D3E9042EE"));
     assert_true(key->valid());
     assert_false(key->expired());
     assert_int_equal(key->expiration(), 0);
@@ -572,7 +625,7 @@ TEST_F(rnp_tests, test_key_expiry_direct_sig)
 {
     /* this test was mainly used to generate test data for cases 10-12 in test_key_validity */
     rnp_key_store_t *secring =
-      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-sec.pgp");
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-sec.pgp", global_ctx);
     assert_true(rnp_key_store_load_from_path(secring, NULL));
     pgp_key_t *key = NULL;
     assert_non_null(key = rnp_tests_key_search(secring, "Alice <alice@rnp>"));
@@ -590,20 +643,19 @@ TEST_F(rnp_tests, test_key_expiry_direct_sig)
     sig.set_keyfp(key->fp());
     sig.set_keyid(key->keyid());
 
-    pgp_password_provider_t pprov = {.callback = string_copy_password_callback,
-                                     .userdata = (void *) "password"};
+    pgp_password_provider_t pprov(string_copy_password_callback, (void *) "password");
     key->unlock(pprov);
-    assert_true(signature_calculate_direct(&key->pkt(), &sig, &key->pkt()));
+    key->sign_direct(key->pkt(), sig, global_ctx);
     key->add_sig(sig, PGP_UID_NONE);
     key->revalidate(*secring);
 
-    /* key is still valid since it is secret */
+    /* key becomsed invalid even since it is secret */
     assert_int_equal(key->expiration(), 1000);
-    assert_true(key->valid());
-    assert_false(key->expired());
+    assert_false(key->valid());
+    assert_true(key->expired());
 
     rnp_key_store_t *pubring =
-      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-pub.pgp");
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-pub.pgp", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     pgp_key_t *pubkey = NULL;
     assert_non_null(pubkey = rnp_tests_key_search(pubring, "Alice <alice@rnp>"));
@@ -611,7 +663,7 @@ TEST_F(rnp_tests, test_key_expiry_direct_sig)
     assert_true(pubkey->valid());
     assert_false(pubkey->expired());
     pgp_key_t *subpub = NULL;
-    assert_non_null(subpub = rnp_tests_get_key_by_id(pubring, "dd23ceb7febeff17", NULL));
+    assert_non_null(subpub = rnp_tests_get_key_by_id(pubring, "dd23ceb7febeff17"));
     assert_int_equal(subpub->expiration(), 0);
     assert_true(subpub->valid());
     assert_false(subpub->expired());
@@ -628,20 +680,21 @@ TEST_F(rnp_tests, test_key_expiry_direct_sig)
     /* add primary userid with smaller expiration date */
     rnp_selfsig_cert_info_t selfsig1 = {};
     const char *            boris = "Boris <boris@rnp>";
-    memcpy(selfsig1.userid, boris, strlen(boris));
+    selfsig1.userid = boris;
     selfsig1.key_expiration = 100;
     selfsig1.primary = true;
-    assert_true(pgp_key_add_userid_certified(key, &key->pkt(), PGP_HASH_SHA256, &selfsig1));
+    key->add_uid_cert(selfsig1, PGP_HASH_SHA256, global_ctx);
     key->revalidate(*secring);
-    /* key is still valid since it is secret, however expiration was changed */
+    /* key becomes invalid even it is secret */
     assert_int_equal(key->expiration(), 100);
-    assert_true(key->valid());
-    assert_false(key->expired());
+    assert_false(key->valid());
+    assert_true(key->expired());
 
     delete secring;
     delete pubring;
 
-    secring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-sec.pgp");
+    secring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-sec.pgp", global_ctx);
     assert_true(rnp_key_store_load_from_path(secring, NULL));
     assert_non_null(key = rnp_tests_key_search(secring, "Alice <alice@rnp>"));
     /* create direct-key signature */
@@ -656,23 +709,24 @@ TEST_F(rnp_tests, test_key_expiry_direct_sig)
     sig.set_keyid(key->keyid());
 
     key->unlock(pprov);
-    assert_true(signature_calculate_direct(&key->pkt(), &sig, &key->pkt()));
+    key->sign_direct(key->pkt(), sig, global_ctx);
     key->add_sig(sig, PGP_UID_NONE);
     key->revalidate(*secring);
     assert_int_equal(key->expiration(), 6);
     /* add primary userid with 0 expiration */
     selfsig1 = {};
-    memcpy(selfsig1.userid, boris, strlen(boris));
+    selfsig1.userid = boris;
     selfsig1.key_expiration = 0;
     selfsig1.primary = true;
-    assert_true(pgp_key_add_userid_certified(key, &key->pkt(), PGP_HASH_SHA256, &selfsig1));
+    key->add_uid_cert(selfsig1, PGP_HASH_SHA256, global_ctx);
     key->revalidate(*secring);
     assert_int_equal(key->expiration(), 6);
 
-    pubring = new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-pub.pgp");
+    pubring =
+      new rnp_key_store_t(PGP_KEY_STORE_GPG, KEYSIG_PATH "alice-sub-pub.pgp", global_ctx);
     assert_true(rnp_key_store_load_from_path(pubring, NULL));
     assert_non_null(pubkey = rnp_tests_key_search(pubring, "Alice <alice@rnp>"));
-    assert_non_null(subpub = rnp_tests_get_key_by_id(pubring, "dd23ceb7febeff17", NULL));
+    assert_non_null(subpub = rnp_tests_get_key_by_id(pubring, "dd23ceb7febeff17"));
     assert_int_equal(subpub->expiration(), 0);
     assert_true(subpub->valid());
     assert_false(subpub->expired());
