@@ -63,7 +63,6 @@ TEST_F(rnp_tests, test_load_v3_keyring_pgp)
     delete key_store;
 
     // load secret keyring and decrypt the key
-
     key_store = new rnp_key_store_t(global_ctx);
 
     assert_rnp_success(init_file_src(&src, "data/keyrings/4/secring.pgp"));
@@ -84,7 +83,11 @@ TEST_F(rnp_tests, test_load_v3_keyring_pgp)
 
     // decrypt the key
     pgp_key_pkt_t *seckey = pgp_decrypt_seckey_pgp(key->rawpkt(), key->pkt(), "password");
+#if defined(ENABLE_IDEA)
     assert_non_null(seckey);
+#else
+    assert_null(seckey);
+#endif
 
     // cleanup
     delete seckey;
@@ -418,7 +421,7 @@ static bool
 load_transferable_key(pgp_transferable_key_t *key, const char *fname)
 {
     pgp_source_t src = {};
-    bool         res = !init_file_src(&src, fname) && !process_pgp_key(&src, *key, false);
+    bool         res = !init_file_src(&src, fname) && !process_pgp_key(src, *key, false);
     src_close(&src);
     return res;
 }
