@@ -27,6 +27,10 @@
 #include "support.h"
 #include "rnp_tests.h"
 #include "file-utils.h"
+#include <librepgp/stream-ctx.h>
+#include "pgp-key.h"
+#include "librepgp/stream-armor.h"
+#include "ffi-priv-types.h"
 
 #ifdef _MSC_VER
 #include "uniwin.h"
@@ -44,13 +48,12 @@
 #include <pgp-key.h>
 #include <fstream>
 #include <vector>
+#include <algorithm>
 
 #ifndef WINSHELLAPI
 #include <ftw.h>
 #endif
 
-extern rng_t global_rng;
-
 #ifdef _WIN32
 int
 setenv(const char *name, const char *value, int overwrite)
@@ -205,17 +208,19 @@ get_tmp()
 static bool
 is_tmp_path(const char *path)
 {
-    char rlpath[PATH_MAX] = {0};
-    char rltmp[PATH_MAX] = {0};
-    if (!realpath(path, rlpath)) {
-        strncpy(rlpath, path, sizeof(rlpath));
-        rlpath[sizeof(rlpath) - 1] = '\0';
+    char *rlpath = realpath(path, NULL);
+    if (!rlpath) {
+        rlpath = strdup(path);
     }
     const char *tmp = get_tmp();
-    if (!realpath(tmp, rltmp)) {
-        strncpy(rltmp, tmp, sizeof(rltmp));
+    char *      rltmp = realpath(tmp, NULL);
+    if (!rltmp) {
+        rltmp = strdup(tmp);
     }
-    return strncmp(rlpath, rltmp, strlen(rltmp)) == 0;
+    bool res = rlpath && rltmp && !strncmp(rlpath, rltmp, strlen(rltmp));
+    free(rlpath);
+    free(rltmp);
+    return res;
 }
 
 /* Recursively remove a directory.
@@ -230,22 +235,22 @@ delete_recursively(const char *path)
 #else
       *path != '/';
 #endif
-    char *fullpath = const_cast<char *>(path);
+    std::string fullpath = path;
     if (relative) {
         char *cwd = getcwd(NULL, 0);
-        fullpath = rnp_compose_path(cwd, path, NULL);
+        fullpath = rnp::path::append(cwd, fullpath);
         free(cwd);
     }
     /* sanity check, we should only be purging things from /tmp/ */
-    assert_true(is_tmp_path(fullpath));
+    assert_true(is_tmp_path(fullpath.c_str()));
 
 #ifdef WINSHELLAPI
     SHFILEOPSTRUCTA fileOp = {};
     fileOp.fFlags = FOF_NOCONFIRMATION;
-    assert_true(strlen(fullpath) < MAX_PATH);
+    assert_true(fullpath.size() < MAX_PATH);
     char newFrom[MAX_PATH + 1];
-    strcpy_s(newFrom, fullpath);
-    newFrom[strlen(fullpath) + 1] = NULL; // two NULLs are required
+    strcpy_s(newFrom, fullpath.c_str());
+    newFrom[fullpath.size() + 1] = NULL; // two NULLs are required
     fileOp.pFrom = newFrom;
     fileOp.pTo = NULL;
     fileOp.wFunc = FO_DELETE;
@@ -255,9 +260,6 @@ delete_recursively(const char *path)
     assert_int_equal(0, SHFileOperationA(&fileOp));
 #else
     nftw(path, remove_cb, 64, FTW_DEPTH | FTW_PHYS);
-    if (*path != '/') {
-        free(fullpath);
-    }
 #endif
 }
 
@@ -350,65 +352,6 @@ make_temp_dir()
 #error Unsupported platform
 #endif
 
-static char *
-directory_from_absolute_file_path(const char *file_path)
-{
-    const char *last_sep = strrchr(file_path, '/');
-    if (!last_sep) {
-        return NULL;
-    }
-
-    size_t file_path_len = (last_sep - file_path);
-    size_t dir_len = file_path_len + 1;
-    char * dir = (char *) calloc(1, dir_len);
-    if (!dir) {
-        return NULL;
-    }
-    strncpy(dir, file_path, file_path_len);
-
-    char *full_dir = realpath(dir, NULL);
-    free(dir);
-    dir = NULL;
-    return full_dir;
-}
-
-static char *
-directory_from_relative_file_path(const char *file_path, const char *reldir)
-{
-    const char *last_sep = strrchr(file_path, '/');
-    if (!last_sep) {
-        return NULL;
-    }
-
-    size_t file_path_len = (last_sep - file_path);
-    size_t dir_len = strlen(reldir) + 1 + file_path_len + 1;
-    char * dir = (char *) calloc(1, dir_len);
-    if (!dir) {
-        return NULL;
-    }
-
-    strncpy(dir, reldir, dir_len);
-    strncat(dir, "/", dir_len);
-    strncat(dir, file_path, file_path_len);
-
-    char *full_dir = realpath(dir, NULL);
-    free(dir);
-    dir = NULL;
-    return full_dir;
-}
-
-char *
-directory_from_file_path(const char *file_path, const char *reldir)
-{
-    if (!file_path) {
-        return NULL;
-    }
-    if (*file_path == '/') {
-        return directory_from_absolute_file_path(file_path);
-    }
-    return directory_from_relative_file_path(file_path, reldir);
-}
-
 bool
 bin_eq_hex(const uint8_t *data, size_t len, const char *val)
 {
@@ -448,15 +391,15 @@ hex2mpi(pgp_mpi_t *val, const char *hex)
 }
 
 bool
-cmp_keyid(const pgp_key_id_t &id, const char *val)
+cmp_keyid(const pgp_key_id_t &id, const std::string &val)
 {
-    return bin_eq_hex(id.data(), id.size(), val);
+    return bin_eq_hex(id.data(), id.size(), val.c_str());
 }
 
 bool
-cmp_keyfp(const pgp_fingerprint_t &fp, const char *val)
+cmp_keyfp(const pgp_fingerprint_t &fp, const std::string &val)
 {
-    return bin_eq_hex(fp.fingerprint, fp.length, val);
+    return bin_eq_hex(fp.fingerprint, fp.length, val.c_str());
 }
 
 int
@@ -473,6 +416,16 @@ test_value_equal(const char *what, const char *expected_value, const uint8_t v[]
     return 0;
 }
 
+void
+test_ffi_init(rnp_ffi_t *ffi)
+{
+    // setup FFI
+    assert_rnp_success(rnp_ffi_create(ffi, "GPG", "GPG"));
+    // load our keyrings
+    assert_true(
+      load_keys_gpg(*ffi, "data/keyrings/1/pubring.gpg", "data/keyrings/1/secring.gpg"));
+}
+
 bool
 mpi_empty(const pgp_mpi_t &val)
 {
@@ -604,7 +557,7 @@ setup_cli_rnp_common(cli_rnp_t *rnp, const char *ks_format, const char *homedir,
     }
 
     /*initialize the basic RNP structure. */
-    return cli_rnp_init(rnp, cfg);
+    return rnp->init(cfg);
 }
 
 void
@@ -689,6 +642,28 @@ string_copy_password_callback(const pgp_password_ctx_t *ctx,
     return true;
 }
 
+void
+unused_getkeycb(rnp_ffi_t   ffi,
+                void *      app_ctx,
+                const char *identifier_type,
+                const char *identifier,
+                bool        secret)
+{
+    assert_true(false);
+}
+
+bool
+unused_getpasscb(rnp_ffi_t        ffi,
+                 void *           app_ctx,
+                 rnp_key_handle_t key,
+                 const char *     pgp_context,
+                 char *           buf,
+                 size_t           buf_len)
+{
+    assert_true(false);
+    return false;
+}
+
 bool
 starts_with(const std::string &data, const std::string &match)
 {
@@ -734,6 +709,15 @@ strip_eol(const std::string &str)
     return str;
 }
 
+std::string
+lowercase(const std::string &str)
+{
+    std::string res = str;
+    std::transform(
+      res.begin(), res.end(), res.begin(), [](unsigned char ch) { return std::tolower(ch); });
+    return res;
+}
+
 static bool
 jso_get_field(json_object *obj, json_object **fld, const std::string &name)
 {
@@ -838,7 +822,34 @@ rnp_tests_get_key_by_id(rnp_key_store_t *keyring, const std::string &keyid, pgp_
     if (binlen > PGP_KEY_ID_SIZE) {
         return NULL;
     }
-    return rnp_key_store_get_key_by_id(keyring, keyid_bin, after);
+    pgp_key_search_t search(PGP_KEY_SEARCH_KEYID);
+    search.by.keyid = keyid_bin;
+    return rnp_key_store_search(keyring, &search, after);
+}
+
+pgp_key_t *
+rnp_tests_get_key_by_grip(rnp_key_store_t *keyring, const std::string &grip)
+{
+    if (!keyring || grip.empty() || !ishex(grip)) {
+        return NULL;
+    }
+    pgp_key_grip_t grip_bin = {};
+    size_t         binlen = rnp::hex_decode(grip.c_str(), grip_bin.data(), grip_bin.size());
+    if (binlen > PGP_KEY_GRIP_SIZE) {
+        return NULL;
+    }
+    return rnp_tests_get_key_by_grip(keyring, grip_bin);
+}
+
+pgp_key_t *
+rnp_tests_get_key_by_grip(rnp_key_store_t *keyring, const pgp_key_grip_t &grip)
+{
+    if (!keyring) {
+        return NULL;
+    }
+    pgp_key_search_t search(PGP_KEY_SEARCH_GRIP);
+    search.by.grip = grip;
+    return rnp_key_store_search(keyring, &search, NULL);
 }
 
 pgp_key_t *
@@ -858,14 +869,14 @@ rnp_tests_get_key_by_fpr(rnp_key_store_t *keyring, const std::string &keyid)
 }
 
 pgp_key_t *
-rnp_tests_key_search(rnp_key_store_t *keyring, const std::string &keyid)
+rnp_tests_key_search(rnp_key_store_t *keyring, const std::string &uid)
 {
-    if (!keyring || keyid.empty()) {
+    if (!keyring || uid.empty()) {
         return NULL;
     }
 
-    pgp_key_search_t srch_userid = {PGP_KEY_SEARCH_USERID};
-    strncpy(srch_userid.by.userid, keyid.c_str(), sizeof(srch_userid.by.userid));
+    pgp_key_search_t srch_userid(PGP_KEY_SEARCH_USERID);
+    strncpy(srch_userid.by.userid, uid.c_str(), sizeof(srch_userid.by.userid));
     srch_userid.by.userid[sizeof(srch_userid.by.userid) - 1] = '\0';
     return rnp_key_store_search(keyring, &srch_userid, NULL);
 }
@@ -999,9 +1010,10 @@ import_keys(rnp_ffi_t ffi, const uint8_t *data, size_t len, uint32_t flags)
 }
 
 bool
-import_all_keys(rnp_ffi_t ffi, const uint8_t *data, size_t len)
+import_all_keys(rnp_ffi_t ffi, const uint8_t *data, size_t len, uint32_t flags)
 {
-    return import_keys(ffi, data, len, RNP_LOAD_SAVE_PUBLIC_KEYS | RNP_LOAD_SAVE_SECRET_KEYS);
+    return import_keys(
+      ffi, data, len, RNP_LOAD_SAVE_PUBLIC_KEYS | RNP_LOAD_SAVE_SECRET_KEYS | flags);
 }
 
 bool
@@ -1015,3 +1027,220 @@ import_sec_keys(rnp_ffi_t ffi, const uint8_t *data, size_t len)
 {
     return import_keys(ffi, data, len, RNP_LOAD_SAVE_SECRET_KEYS);
 }
+
+std::vector<uint8_t>
+export_key(rnp_key_handle_t key, bool armored, bool secret)
+{
+    uint32_t flags = RNP_KEY_EXPORT_SUBKEYS;
+    if (armored) {
+        flags = flags | RNP_KEY_EXPORT_ARMORED;
+    }
+    if (secret) {
+        flags = flags | RNP_KEY_EXPORT_SECRET;
+    } else {
+        flags = flags | RNP_KEY_EXPORT_PUBLIC;
+    }
+    rnp_output_t output = NULL;
+    rnp_output_to_memory(&output, 0);
+    rnp_key_export(key, output, flags);
+    size_t   len = 0;
+    uint8_t *buf = NULL;
+    rnp_output_memory_get_buf(output, &buf, &len, false);
+    std::vector<uint8_t> res(buf, buf + len);
+    rnp_output_destroy(output);
+    return res;
+}
+
+void
+dump_key_stdout(rnp_key_handle_t key, bool secret)
+{
+    auto pub = export_key(key, true, false);
+    printf("%.*s", (int) pub.size(), (char *) pub.data());
+    if (!secret) {
+        return;
+    }
+    auto sec = export_key(key, true, true);
+    printf("%.*s", (int) sec.size(), (char *) sec.data());
+}
+
+bool
+write_transferable_key(pgp_transferable_key_t &key, pgp_dest_t &dst, bool armor)
+{
+    pgp_key_sequence_t keys;
+    keys.keys.push_back(key);
+    return write_transferable_keys(keys, &dst, armor);
+}
+
+bool
+write_transferable_keys(pgp_key_sequence_t &keys, pgp_dest_t *dst, bool armor)
+{
+    std::unique_ptr<rnp::ArmoredDest> armdst;
+    if (armor) {
+        pgp_armored_msg_t msgtype = PGP_ARMORED_PUBLIC_KEY;
+        if (!keys.keys.empty() && is_secret_key_pkt(keys.keys.front().key.tag)) {
+            msgtype = PGP_ARMORED_SECRET_KEY;
+        }
+        armdst = std::unique_ptr<rnp::ArmoredDest>(new rnp::ArmoredDest(*dst, msgtype));
+        dst = &armdst->dst();
+    }
+
+    for (auto &key : keys.keys) {
+        /* main key */
+        key.key.write(*dst);
+        /* revocation and direct-key signatures */
+        for (auto &sig : key.signatures) {
+            sig.write(*dst);
+        }
+        /* user ids/attrs and signatures */
+        for (auto &uid : key.userids) {
+            uid.uid.write(*dst);
+            for (auto &sig : uid.signatures) {
+                sig.write(*dst);
+            }
+        }
+        /* subkeys with signatures */
+        for (auto &skey : key.subkeys) {
+            skey.subkey.write(*dst);
+            for (auto &sig : skey.signatures) {
+                sig.write(*dst);
+            }
+        }
+    }
+    return !dst->werr;
+}
+
+bool
+check_uid_valid(rnp_key_handle_t key, size_t idx, bool valid)
+{
+    rnp_uid_handle_t uid = NULL;
+    if (rnp_key_get_uid_handle_at(key, idx, &uid)) {
+        return false;
+    }
+    bool val = !valid;
+    rnp_uid_is_valid(uid, &val);
+    rnp_uid_handle_destroy(uid);
+    return val == valid;
+}
+
+bool
+check_uid_primary(rnp_key_handle_t key, size_t idx, bool primary)
+{
+    rnp_uid_handle_t uid = NULL;
+    if (rnp_key_get_uid_handle_at(key, idx, &uid)) {
+        return false;
+    }
+    bool prim = !primary;
+    rnp_uid_is_primary(uid, &prim);
+    rnp_uid_handle_destroy(uid);
+    return prim == primary;
+}
+
+bool
+check_key_valid(rnp_key_handle_t key, bool validity)
+{
+    bool valid = !validity;
+    if (rnp_key_is_valid(key, &valid)) {
+        return false;
+    }
+    return valid == validity;
+}
+
+uint32_t
+get_key_expiry(rnp_key_handle_t key)
+{
+    uint32_t expiry = (uint32_t) -1;
+    rnp_key_get_expiration(key, &expiry);
+    return expiry;
+}
+
+size_t
+get_key_uids(rnp_key_handle_t key)
+{
+    size_t count = (size_t) -1;
+    rnp_key_get_uid_count(key, &count);
+    return count;
+}
+
+bool
+check_sub_valid(rnp_key_handle_t key, size_t idx, bool validity)
+{
+    rnp_key_handle_t sub = NULL;
+    if (rnp_key_get_subkey_at(key, idx, &sub)) {
+        return false;
+    }
+    bool valid = !validity;
+    rnp_key_is_valid(sub, &valid);
+    rnp_key_handle_destroy(sub);
+    return valid == validity;
+}
+
+rnp_key_handle_t
+bogus_key_handle(rnp_ffi_t ffi)
+{
+    rnp_key_handle_t handle = (rnp_key_handle_t) calloc(1, sizeof(*handle));
+    handle->ffi = ffi;
+    handle->pub = NULL;
+    handle->sec = NULL;
+    handle->locator.type = PGP_KEY_SEARCH_KEYID;
+    return handle;
+}
+
+bool
+sm2_enabled()
+{
+    bool enabled = false;
+    if (rnp_supports_feature(RNP_FEATURE_PK_ALG, "SM2", &enabled)) {
+        return false;
+    }
+    return enabled;
+}
+
+bool
+aead_eax_enabled()
+{
+    bool enabled = false;
+    if (rnp_supports_feature(RNP_FEATURE_AEAD_ALG, "EAX", &enabled)) {
+        return false;
+    }
+    return enabled;
+}
+
+bool
+aead_ocb_enabled()
+{
+    bool enabled = false;
+    if (rnp_supports_feature(RNP_FEATURE_AEAD_ALG, "OCB", &enabled)) {
+        return false;
+    }
+    return enabled;
+}
+
+bool
+twofish_enabled()
+{
+    bool enabled = false;
+    if (rnp_supports_feature(RNP_FEATURE_SYMM_ALG, "Twofish", &enabled)) {
+        return false;
+    }
+    return enabled;
+}
+
+bool
+idea_enabled()
+{
+    bool enabled = false;
+    if (rnp_supports_feature(RNP_FEATURE_SYMM_ALG, "IDEA", &enabled)) {
+        return false;
+    }
+    return enabled;
+}
+
+bool
+brainpool_enabled()
+{
+    bool enabled = false;
+    if (rnp_supports_feature(RNP_FEATURE_CURVE, "brainpoolP256r1", &enabled)) {
+        return false;
+    }
+    return enabled;
+}
