@@ -122,25 +122,44 @@ TEST_F(rnp_tests, test_rnpcfg_get_expiration)
     timeinfo->tm_min = 0;
     timeinfo->tm_sec = 0;
     rawtime = mktime(timeinfo);
-    auto exp =
-      fmt("%d-%02d-%02d", timeinfo->tm_year + 1900, timeinfo->tm_mon + 1, timeinfo->tm_mday);
+    auto    year = timeinfo->tm_year + 1900;
+    auto    mon = timeinfo->tm_mon + 1;
+    auto    day = timeinfo->tm_mday;
+    rnp_cfg cfg;
+    cfg.set_str("expiry-", fmt("%d-%02d-%02d", year, mon, day));
+    cfg.set_str("expiry/", fmt("%d/%02d/%02d", year, mon, day));
+    cfg.set_str("expiry.", fmt("%d.%02d.%02d", year, mon, day));
+
     uint32_t raw_expiry = 0;
-    assert_int_equal(get_expiration(exp.c_str(), &raw_expiry), 0);
+    assert_true(cfg.get_expiration("expiry-", raw_expiry));
     assert_int_equal(raw_expiry, rawtime - basetime);
-    assert_int_equal(get_expiration("2100-01-01", &raw_expiry), 0);
-    assert_int_equal(get_expiration("2024-02-29", &raw_expiry), 0);
+    assert_true(cfg.get_expiration("expiry/", raw_expiry));
+    assert_int_equal(raw_expiry, rawtime - basetime);
+    assert_true(cfg.get_expiration("expiry.", raw_expiry));
+    assert_int_equal(raw_expiry, rawtime - basetime);
+    cfg.set_str("expiry", "2100-01-01");
+    assert_true(cfg.get_expiration("expiry", raw_expiry));
+    assert_int_not_equal(raw_expiry, rawtime - basetime);
+    cfg.set_str("expiry", "2124-02-29");
+    assert_true(cfg.get_expiration("expiry", raw_expiry));
     /* date in a past */
-    assert_int_not_equal(get_expiration("2000-02-29", &raw_expiry), 0);
+    cfg.set_str("expiry", "2000-02-29");
+    assert_false(cfg.get_expiration("expiry", raw_expiry));
+    cfg.set_str("expiry", "2400-02-29");
     if ((sizeof(time_t) > 4)) {
         /* date is correct, but overflows 32 bits */
-        assert_int_not_equal(get_expiration("2400-02-29", &raw_expiry), 0);
+        assert_false(cfg.get_expiration("expiry", raw_expiry));
     } else {
         /* for 32-bit time_t we return INT32_MAX for all dates beyond the y2038 */
-        assert_int_equal(get_expiration("2400-02-29", &raw_expiry), 0);
+        assert_true(cfg.get_expiration("expiry", raw_expiry));
         assert_int_equal(raw_expiry, INT32_MAX - basetime);
     }
-    assert_int_not_equal(get_expiration("2100-02-29", &raw_expiry), 0);
-    assert_int_not_equal(get_expiration("4294967296", &raw_expiry), 0);
-    assert_int_not_equal(get_expiration("2037-02-29", &raw_expiry), 0);
-    assert_int_not_equal(get_expiration("2037-13-01", &raw_expiry), 0);
+    cfg.set_str("expiry", "2100-02-29");
+    assert_false(cfg.get_expiration("expiry", raw_expiry));
+    cfg.set_str("expiry", "4294967296");
+    assert_false(cfg.get_expiration("expiry", raw_expiry));
+    cfg.set_str("expiry", "2037-02-29");
+    assert_false(cfg.get_expiration("expiry", raw_expiry));
+    cfg.set_str("expiry", "2037-13-01");
+    assert_false(cfg.get_expiration("expiry", raw_expiry));
 }
